+{
   locale_version => 1.01,
   entry => <<'ENTRY', # for DUCET v6.3.0
0063 0068 ; [.1602.0020.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0043 0068 ; [.1602.0020.0007] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.1602.0020.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
0067 0062 ; [.168E.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER B>
0047 0062 ; [.168E.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER B>
0047 0042 ; [.168E.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER B>
0067 0068 ; [.168F.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER H>
0047 0068 ; [.168F.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER H>
0047 0048 ; [.168F.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER H>
0067 0077 ; [.1690.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER W>
0047 0077 ; [.1690.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER W>
0047 0057 ; [.1690.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER W>
1ECB      ; [.16CA.0020.0002] # LATIN SMALL LETTER I WITH DOT BELOW
0069 0323 ; [.16CA.0020.0002] # LATIN SMALL LETTER I WITH DOT BELOW
1ECA      ; [.16CA.0020.0008] # LATIN CAPITAL LETTER I WITH DOT BELOW
0049 0323 ; [.16CA.0020.0008] # LATIN CAPITAL LETTER I WITH DOT BELOW
006B 0070 ; [.16FC.0020.0002] # <LATIN SMALL LETTER K, LATIN SMALL LETTER P>
004B 0070 ; [.16FC.0020.0007] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER P>
004B 0050 ; [.16FC.0020.0008] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER P>
006B 0077 ; [.16FD.0020.0002] # <LATIN SMALL LETTER K, LATIN SMALL LETTER W>
004B 0077 ; [.16FD.0020.0007] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER W>
004B 0057 ; [.16FD.0020.0008] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER W>
1E45      ; [.174C.0020.0002] # LATIN SMALL LETTER N WITH DOT ABOVE
006E 0307 ; [.174C.0020.0002] # LATIN SMALL LETTER N WITH DOT ABOVE
1E44      ; [.174C.0020.0008] # LATIN CAPITAL LETTER N WITH DOT ABOVE
004E 0307 ; [.174C.0020.0008] # LATIN CAPITAL LETTER N WITH DOT ABOVE
006E 0077 ; [.174D.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER W>
004E 0077 ; [.174D.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER W>
004E 0057 ; [.174D.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER W>
006E 0079 ; [.174E.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
004E 0079 ; [.174E.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
004E 0059 ; [.174E.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
1ECD      ; [.176E.0020.0002] # LATIN SMALL LETTER O WITH DOT BELOW
006F 0323 ; [.176E.0020.0002] # LATIN SMALL LETTER O WITH DOT BELOW
1ECC      ; [.176E.0020.0008] # LATIN CAPITAL LETTER O WITH DOT BELOW
004F 0323 ; [.176E.0020.0008] # LATIN CAPITAL LETTER O WITH DOT BELOW
1ED9      ; [.176E.0020.0002][.0000.0027.0002] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1ED8      ; [.176E.0020.0008][.0000.0027.0002] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1EE3      ; [.176E.0020.0002][.0000.003F.0002] # LATIN SMALL LETTER O WITH HORN AND DOT BELOW
1EE2      ; [.176E.0020.0008][.0000.003F.0002] # LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW
0073 0068 ; [.17F0.0020.0002] # <LATIN SMALL LETTER S, LATIN SMALL LETTER H>
0053 0068 ; [.17F0.0020.0007] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER H>
0053 0048 ; [.17F0.0020.0008] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER H>
1EE5      ; [.1833.0020.0002] # LATIN SMALL LETTER U WITH DOT BELOW
0075 0323 ; [.1833.0020.0002] # LATIN SMALL LETTER U WITH DOT BELOW
1EE4      ; [.1833.0020.0008] # LATIN CAPITAL LETTER U WITH DOT BELOW
0055 0323 ; [.1833.0020.0008] # LATIN CAPITAL LETTER U WITH DOT BELOW
1EF1      ; [.1833.0020.0002][.0000.003F.0002] # LATIN SMALL LETTER U WITH HORN AND DOT BELOW
1EF0      ; [.1833.0020.0008][.0000.003F.0002] # LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW
ENTRY
};
